/*
 * Copyright (C) 2017  CZ.NIC, z.s.p.o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *  @file automatic_keyset_management_i.hh
 *  automatic keyset management corba impl header
 */

//pregenerated by $> omniidl -bcxx -Wba -Wbexample -C./src/corba ~/workspace_18680/enum/idl/idl/RecordStatement.idl

#ifndef AUTOMATIC_KEYSET_MANAGEMENT_I_HH_E1AE9A8DDC2B4A6F863A562B01D09774
#define AUTOMATIC_KEYSET_MANAGEMENT_I_HH_E1AE9A8DDC2B4A6F863A562B01D09774

#include "src/corba/AutomaticKeysetManagement.hh"
#include "src/fredlib/logger_client.h"

#include <boost/shared_ptr.hpp>

#include <string>
#include <set>

namespace Fred {
namespace AutomaticKeysetManagement {

class AutomaticKeysetManagementImpl; // PIMPL class

} // namespace Fred::AutomaticKeysetManagement
} // namespace Fred

namespace Registry {
namespace AutomaticKeysetManagement {

class Server_i: public POA_Registry::AutomaticKeysetManagement::AutomaticKeysetManagementIntf {
public:
    Server_i(
            const std::string& _server_name,
            const std::string& _automatically_managed_keyset_prefix,
            const std::string& _automatically_managed_keyset_registrar,
            const std::string& _automatically_managed_keyset_tech_contact,
            const std::set<std::string>& _automatically_managed_keyset_zones,
            bool _disable_notifier,
            Fred::Logger::LoggerClient& _logger_client);

    // methods corresponding to defined IDL attributes and operations
    NameserverDomainsSeq* get_nameservers_with_insecure_automatically_managed_domain_candidates();

    NameserverDomainsSeq* get_nameservers_with_secure_automatically_managed_domain_candidates();

    NameserverDomainsSeq* get_nameservers_with_automatically_managed_domains();

    void turn_on_automatic_keyset_management_on_insecure_domain(
            ::CORBA::ULongLong domain_id,
            const Nsset& current_nsset,
            const Keyset& new_keyset);

    void turn_on_automatic_keyset_management_on_secure_domain(
            ::CORBA::ULongLong domain_id,
            const Keyset& new_keyset);

    void update_automatically_managed_keyset_of_domain(
            ::CORBA::ULongLong domain_id,
            const Keyset& new_keyset);

    EmailAddressSeq* get_email_addresses_by_domain_id(
            ::CORBA::ULongLong domain_id);

private:
    // Make sure all instances are built on the heap by making the
    // destructor non-public
    virtual ~Server_i();
    // do not copy
    const boost::shared_ptr<Fred::AutomaticKeysetManagement::AutomaticKeysetManagementImpl> impl_;
    Server_i(const Server_i&);//no body
    Server_i& operator= (const Server_i&);//no body
};

} // namespace Registry::AutomaticKeysetManagement
} // namespace Registry

#endif
